using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

using System.Xml.Serialization;
using SharpCharts.Base.Common;
using SharpCharts.Base.Data;
using OptionsOracle.Calc.Options;

namespace SharpCharts.Base.MarketWatch
{

   public enum GreekType
   {
   		ImpliedVolatility,
        Value,
        Delta,
        Gamma,
        Vega,
        Theta
    }


    public class OptionsOracle : ScannerBase
    {
    
    	private double underlyingValue;
    	
    
       private OptionPricingModel optionPricingModel = null;
       
        public OptionsOracle()
        {
            this.ExpiryDate = DateTime.Now;
            this.Instrument1 = string.Format(System.Globalization.CultureInfo.InvariantCulture, "NIFTY {0:MM yyyy} NSE", DateTime.Now);
        	this.InstrumentCommand = new RelayCommand<Instrument>(OnInstrument);
        }
        
	public override string HeaderString
	{
		get { return this.GreekType.ToString(); }
	}
	
	public override string DisplayName
	{
		get { return HeaderString; }
	} 
        
        
        protected override void OnStateChange()
        {
            switch (base.State)
            {
                case SharpCharts.Base.SharpScript.State.Finalize:
                    break;
                case SharpCharts.Base.SharpScript.State.Initialize:

                    AddInstrument(this.Underlying.DisplayName);
                    break;
            }
        }
        
        
        protected override void OnLevel1(int idx, Data.RealTimeDataEventArgs e)
        {
        	if (e.MarketDataType != MarketDataType.Last)
        		return;
                                  	
            if (idx == 1)
            {
            	this.underlyingValue = e.Price;
            	return;
            }
            
            if (this.underlyingValue <= 0.0d)
            	return;
        	
			//Sould be an option        	
            if (e.Instrument.InstrumentDefination.InstrumentType != InstrumentType.Options)
            {
            	return;
            }
               	
           	//calculate the greeks
           	double timeRemaining = (this.ExpiryDate.Date.AddHours(15).AddMinutes(30) - DateTime.Now).TotalDays / 365.0d;
               	
               	               	
           	if (this.optionPricingModel == null)
           	{
           		this.optionPricingModel = new BlackScholes();
           	}
               	
            double iv = this.optionPricingModel.ImpliedVolatility(e.Instrument.OptionType, e.Price, underlyingValue, e.Instrument.StrikePrice, 
            	this.InterestRate, this.Dividend, timeRemaining);
               	
            
            
           	switch (this.GreekType)
           	
           	{
           		case GreekType.ImpliedVolatility:
           			base.DisplayValue = Math.Round(iv * 100, this.Precision);
           			break;
           		case GreekType.Value:
           			double delta;
           			base.DisplayValue = Math.Round(this.optionPricingModel.TheoreticalOptionPrice(e.Instrument.OptionType, this.underlyingValue, e.Instrument.StrikePrice, this.InterestRate, this.Dividend, iv, timeRemaining, out delta), this.Precision);
               		break;
               	case GreekType.Delta:
               		base.DisplayValue = Math.Round(this.optionPricingModel.Delta(e.Instrument.OptionType, this.underlyingValue, e.Instrument.StrikePrice, this.InterestRate, this.Dividend, iv, timeRemaining), this.Precision);
               		break;
               	case GreekType.Gamma:
               		base.DisplayValue = Math.Round(this.optionPricingModel.Gamma(underlyingValue, e.Instrument.StrikePrice, this.InterestRate, this.Dividend, iv, timeRemaining), this.Precision);
               		break;
               	case GreekType.Vega:
               		base.DisplayValue = Math.Round(this.optionPricingModel.Vega(underlyingValue, e.Instrument.StrikePrice, this.InterestRate, this.Dividend, iv, timeRemaining) / 100.0d, this.Precision);
               		break;
               	case GreekType.Theta:
               		base.DisplayValue = Math.Round(this.optionPricingModel.Theta(e.Instrument.OptionType, underlyingValue, e.Instrument.StrikePrice, this.InterestRate, this.Dividend, iv, timeRemaining) / 365.0d, this.Precision);
               		break;
               	}
               
        }
        
        
        /*
        private double GetLast(Instrument contact)
        {
        	  if (contact == null)
        	  	return 0.0d;
        	  
                if (contact.RealTimeData == null || contact.RealTimeData.Last == null)
                    return 0.0d;

                return contact.RealTimeData.Last.Price;
          }
          */
        private void OnInstrument(Instrument instrument)
        {
        	this.Underlying = instrument;        	
        }
          
       public GreekType[] GreekTypes
       {
       	get { return (GreekType[])Enum.GetValues(typeof(GreekType)); }
       }
        
               
        private double interestRate = 0.06;
        
        public double InterestRate 
        {
        	get { return interestRate; }
        	set { interestRate = Math.Min(Math.Max(0.0d, value), 1); }
        }
        
        private double dividend = 0.0d;
        
        public double Dividend
        {
        	get { return dividend; }
        	set { dividend = Math.Min(Math.Max(0.0d, value), 1); }
        }
        
        private int precision = 4;
        
        public int Precision
        {
        	get { return precision; }
        	set { precision = Math.Max(2, value); }
        }
        
        
        private GreekType greekType = GreekType.Value;
        
        public GreekType GreekType
        {
        	get { return greekType; }
        	set 
        	{
	        	greekType = value; 
	       		base.NotifyPropertyChanged("HeaderString");
	       		base.NotifyPropertyChanged("DisplayName");
	       }
        }
        
        [XmlIgnore]
        public ICommand InstrumentCommand { get; private set; }
        
        
        
        private Instrument underlying = null;
		[XmlIgnore]
        public Instrument Underlying
		{
			get { return underlying; }
			set 
			{
				underlying = value; 
				NotifyPropertyChanged("Underlying");
			}
		}    
		
		private string instrument1;
		
		public string Instrument1
		{
			get
			{
				if (this.Underlying == null)
					return instrument1;
				return this.Underlying.DisplayName;
			}
			set
			{
				this.Underlying = null;
				instrument1 = value;
				if (!string.IsNullOrWhiteSpace(instrument1))
				{
					Instrument.TryParse(instrument1, out this.underlying);
				}
			}
		}
		

		public DateTime ExpiryDate { get; set; }
        
        
    }
}